;----------------------------------------------------------------------------
; This startup code for the EVM
; Modified from the G56K y-space example
; By Johan Forrer, January, 1997
;---------------------------------------------------------------------------- 
	opt so,xr
	page 132,66,3,3

	nolist
	include 'ioequlc'
	include 'intequlc'
	list

TOP_OF_MEMORY   equ     $3FFF

;----------------------------------------------------------------------------
; EVM equates
;----------------------------------------------------------------------------
xtal 	equ 	40000000		; Early EVM's use 40Mhz xtal
baud	equ	9600
buflen	equ	1024
;-----------------------------------------------------------------------------
; Define the PLL multiplier
;-----------------------------------------------------------------------------
	if (xtal==32000000)
ClockMult equ	$7          		; set PLL 8x
	endif
	if (xtal==40000000)
ClockMult equ	$9          		; set PLL 10x
	endif
	if (xtal==66000000)		; NOT DIVISIBLE BY 4 !!! (16.5)
ClockMult equ	$0F			; set PLL 16x
	endif
	if (xtal==68000000)
ClockMult equ	$10			; set PLL 17x
	endif
	if (xtal==80000000)
ClockMult equ   $12			; set PLL 20x
	endif

;----------------------------------------------------------------------------
; LEONID equates 
;----------------------------------------------------------------------------
hbeat	equ	72.0		; heartbeat blinking rate (pulses/min)

;-----------------------------------------------------------------------------
; LEONID Flags 
;-----------------------------------------------------------------------------
rempty	equ	0		; SCI flags
rfull	equ	1
xempty	equ	2
xfull	equ	3
timer	equ	4		; timer flag
scmode	equ	5		; serial communication mode

;----------------------------------------------------------------------------
; LEONID macros 
;----------------------------------------------------------------------------
; macro for green LED handling
copled	macro	mode
	b\mode	#14,x:m_pbd
	b\mode	#14,y:<flags_mirror	; reflect this change
	endm

; macro for red LED handling
cmdled	macro	mode
	b\mode	#13,x:m_pbd
	b\mode	#13,y:<flags_mirror	; reflect this change
	endm

;---------------------------------------------------------------------------
;       This section should be loaded at P:0 that way reset will
;       cause a jump to main.

;       The reset section is located at p:0 in the mapfile. This section is
;       intended to hold system reset code. This code should jump to start
;       when it is done.
;----------------------------------------------------------------------------
	section reset

; Reset vector
	org	p:i_reset
	jmp     F__start

; Stack error interrupt
	jsr     Fabort

; Trace interrupt
	jsr     Fabort

; SWI
	jsr     Fabort

; IRQA
	jsr     Fabort

; IRQB
	jsr     Fabort

; SSI interrupts (in CODEC.ASM)
	jsr     ssi_rx_isr              ;SSI RX
	jsr     ssi_rx_isr              ;SSI RX w/Exception
	jsr     ssi_tx_isr              ;SSI TX
	jsr     ssi_tx_isr              ;SSI TX w/Exception

; SCI receive interrupt
	jsr     Fabort

; SCI receive interrupt with errors
	jsr     Fabort

; SCI transmitter interrupt
	jsr     Fabort

; SCI idle interrupt
	jsr     Fabort

; SCI timer interrupt
	jsr	sci_tim

;NMI Interrupt
	jsr     Fabort

; Host port is not used	(0020-003A)
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort
	jsr     Fabort

; Timer
	jsr     Fabort

; Illegal instruction interrupt
	jsr     Fabort

	endsec

;---------------------------------------------------------------------------	
	section crt0

	org     y:

; The following variables are used for dynamic memory allocation
; __stack_safty: Since dynamic memory and the stack grow towards each other 
; This constant tells brk and sbrk what the minimum amount of space should
; be left between the top of stack during the brk or sbrk call and the end
; of any allocated memory.
; __mem_limit: a constant telling brk and sbrk where the end of available 
; memory is.
; __break: pointer to the next block of memory that can be allocated
; The heap may be moved by changing the initial value of __break.
; This is the base of the heap.
; __y_size: the base of dynamic memory.
; errno: error type: set by some libraries
; __max_signal the last possible signal.

	global  Ferrno
Ferrno          dc      $0

	global F__stack_safety
F__stack_safety dc      1024

	global F__mem_limit
F__mem_limit    dc      TOP_OF_MEMORY

	global F__break
F__break        dc      TOP_OF_MEMORY

	global F__y_size
F__y_size       dc      DSIZE

	global  F__max_signal
F__max_signal   dc      $3e     ; what the quint monitor.


;-----------------------------------------------------------------------------
;                  --------  STARTUP CODE  ---------
;-----------------------------------------------------------------------------
	org     p:
F__start
	global  F__start


;       To change the base of the stack, change the value loaded into the
;       stack pointer here.

	and     #$f3,mr		; Status register=MR|CCR
	and     #$bf,ccr
	move    y:F__y_size,r6  ; initialize the stack pointer
	 
;-----------------------------------------------------------------------------
;      ---- EVM56002-specific Startup code -----
;-----------------------------------------------------------------------------
 	movep   #$261000+ClockMult,x:PLL ; set PLL
	
	movep   #$0000,x:m_bcr          ; zero wait states for ext. memory
	ori     #3,mr                   ; disable interrupts
	movec   #0,sp
	move    #0,omr                  ; single chip mode

; initialize SCI
	movep	#$2b02,x:m_scr	    	; 8,n,1
	movep	#(xtal+2*16*baud)/(2*2*16*baud)-1,x:m_sccr ; round baud

; initialize port B
	movep	#$0000,x:m_pbc	    	; port B as general purpose port
	movep	#$60ff,x:m_pbddr	; PB0-PB7,PB13 and PB14 as outputs
	movep	#$0000,x:m_pbd

; initialize port C
	movep	#$0003,x:m_pcc	    	; We are using TXD,RXD

					; pc2 (D/C~), pc3 (DCD), 
	movep	#$1C,x:m_pcddr	    	; pc4 (C Reset) as outputs

	movep   #0,x:m_pcd		; init PC4,PC3,PC2=0

	move	#@cvi(30/hbeat*baud),n3 ; SCI timer system 
	move	n3,y:n3_sci		; (hearbeat rate 72 pulses/min)
	move	#-1,m3
	move	m3,y:m3_sci

	clr	a
	move	a,y:<timchg
	move	a,y:<timcnt
	move	a,y:<flags
	move	a,y:<flags_mirror

; start sci timer interrupts, just to flash the RED LED
	movep	#$8000,x:m_ipr	    	; SCI=IPL1
	andi	#$fc,mr 		; unmask interrupts

	wait
; flash red LED once to show all is OK  
	cmdled	set
	move	#>@cvi(1.0*baud),x0     ; set timer
	jsr	stimer
_wchr	wait
	move	y:<flags,a
	jset	#timer,a,_wchr	; wait for timer
	cmdled	clr

;---------------------------------------------------------------------------
;      Initialize CS4215 CODEC
;---------------------------------------------------------------------------
	ori     #3,mr                   ; disable interrupts
	move    #RX_BUFF_BASE,x0
	move    x0,y:<RX_PTR             ; Initialize the rx pointer
	move    #TX_BUFF_BASE,x0
	move    x0,y:<TX_PTR             ; Initialize the tx pointer
	jsr     codec_init

	move    #TONE_OUTPUT,y0         ; headphones, line out, 
					; mute spkr, no attn.
	move    y0,y:<TX_BUFF_BASE+2
	move    #TONE_INPUT,y0          ; no input gain, monitor mute
	move    y0,y:<TX_BUFF_BASE+3

	movep   #$fb00,x:m_crb          ; Enable the SSI interrupts.

; IRQA/B not used for the moment -----
;;	movep	#$301b,x:m_ipr		; IRQA/IRQB/SSI are level 3 interrupts.
					; and are LEVEL sensitive
	movep	#$b000,x:m_ipr	    	; SSI=IPL2, SCI=IPL1

;_loop_1				; A bit of test code 
;	jset    #2,x:m_sr,*           	 ; Wait for frame sync to pass.
;	jclr    #2,x:m_sr,*            	; Wait for frame sync.
;
;	move    y:<RX_BUFF_BASE,a        ; Load the left channel input.
;	move    y:<RX_BUFF_BASE+1,b      ; Load the right channel input.
;	
;	move    a,y:<TX_BUFF_BASE        ; Put value in left channel tx.
;	move    b,y:<TX_BUFF_BASE+1      ; Put value in right channel tx.  
;	
;	jmp     _loop_1                  ; Loop back.
;-----------------------------------------------------------------------------
; End of EVM startup code
;-----------------------------------------------------------------------------

	xref    F__init_constructor                 
	jsr     F__init_constructor            
  
	move    #$0,r0          ; funny value to terminate a backtrace.

	jsr     Fmain           ; run user program

	jsr     Fexit           ; when exit ( ) isn't explicitly called,
				; call it.
F__crt0_end
	global  F__crt0_end
	stop                    ; all done

	endsec

;----------------------------------------------------------------------------
	section crt0__fp_shift
	org     y:

;
; floating point table setup
;

	global  F__fp_shift
F__fp_shift     dc      $800000,$c00000,$e00000,$f00000,$f80000,$fc0000
		dc      $fe0000,$ff0000,$ff8000,$ffc000,$ffe000,$fff000
		dc      $fff800,$fffc00,$fffe00,$ffff00,$ffff80,$ffffc0
		dc      $ffffe0,$fffff0,$fffff8,$fffffc,$fffffe

	endsec

;----------------------------------------------------------------------------
	section time_counter
	org     y:
F__time
	global  F__time
	dc      0
	endsec
;----------------------------------------------------------------------------

	section CODEC_data
	org     y:
;----------------------------------------------------------------------------
;;NOTE: These are circular buffers; MUST be on a modulo 4 boundary !!
;--------------------------------------------------------------------------- 
RX_BUFF_BASE    equ     *
	global RX_BUFF_BASE
		dsm	4	; Circular
;RX_data_1_2     ds      1       ;data time slot 1/2 for RX ISR
;RX_data_3_4     ds      1       ;data time slot 3/4 for RX ISR
;RX_data_5_6     ds      1       ;data time slot 5/6 for RX ISR
;RX_data_7_8     ds      1       ;data time slot 7/8 for RX ISR

TX_BUFF_BASE    equ     *
	global TX_BUFF_BASE
		dsm	4	; Circular
;TX_data_1_2     ds      1       ;data time slot 1/2 for TX ISR
;TX_data_3_4     ds      1       ;data time slot 3/4 for TX ISR
;TX_data_5_6     ds      1       ;data time slot 5/6 for TX ISR
;TX_data_7_8     ds      1       ;data time slot 7/8 for TX ISR
;----------------------------------------------------------------------------

RX_PTR          
	global RX_PTR
	ds      1       ; Pointer for rx buffer
TX_PTR          
	global TX_PTR
	ds      1       ; Pointer for tx buffer

timcnt
	global  timcnt
	ds	1
timval
	global  timval
	ds	1
timchg
	global  timchg
	ds	1
save_a0
	global  save_a0
	ds	1
save_a1
	global  save_a1
	ds	1
save_a2
	global  save_a2
	ds	1
save_x0
	global  save_x0
	ds	1
save_x1
	global  save_x1
	ds	1
save_r3
	global  save_r3
	ds	1
save_m3
	global  save_m3
	ds	1
save_n3
	global  save_n3
	ds	1
m3_sci
	global  m3_sci
	ds	1
n3_sci
	global  n3_sci
	ds	1
flags
	global  flags
	ds	1
flags_mirror
	global  flags_mirror		; used for LED display purposes
	ds	1
r0_save
	global  r0_save
	ds	1
m0_save
	global  m0_save
	ds	1
	endsec

;----------------------------------------------------------------------------
	section io_primitives
	org     p:

	nop
F__send
	global  F__send
	rts
	nop

	nop
F__receive
	global  F__receive
	rts
	nop

	endsec

;----------------------------------------------------------------------------
; The following section, dummy_call, is used for doing command-line
; function calls on the ADS and simulator.  If you aren't going to use
; those features, it can be removed.

; The host software puts r0 into y:(r6), and then increments r6.
; Then, the current PC is placed into y:(r6) and again r6 is
; incremented.  The new value of r6 is then moved into r0.  If the
; function that the user is calling from the command line returns a
; structure or union, the size of the structure or union is added to
; r6 (making room for it locally), and the value in r0 is moved into
; a1 (since r0 happened to be pointing to the beginning of this local
; structure).  The arguments to the function are then placed on the
; stack starting at y:(r6), with r6 being incremented accordingly.
; Next, the address of dummy_call is placed into the PC.  The address
; of the target function that is being called is then patched in at
; y:(pc+1) (since the jsr is two words and the second word is the
; destination address).
;
; Following this the device is started and a ``finish'' (essentially)
; is done on dummy_call (which will then call the target function).
; If the function call is interrupted by the host software, the
; ADS/simulator is left in a state where the user can step out of the
; called function (because of the prologue code which is after the jsr
; in dummy_call).
;
; The prologue code decrements r0 to where the return address has been
; stored, moves that return address into ssh, and then decrements r0
; again.  At this point r0 has the address that the old r6 had, so we
; move r0 into r6.  Following that we test a (the return value) and
; then move from y:(r0) into r0, which restores the old r0.  Finally,
; we return to our caller.

	section dummy_call

	org     p:

	global  dummy_call
dummy_call
	jsr     >dummy_call
	move    (r0)-
	move    y:(r0)-,ssh
	move    r0,r6
	tst     a       y:(r0),r0
	rts

	endsec
 
	section init_table
	org p:
	xdef    F__begin_init_table
F__begin_init_table
	dc (-1)
 
	endsec
 
;----------------------------------------------------------------------------
	section cinit
	org     p:
	xref  F__begin_init_table
	xdef  F__init_constructor
F__init_constructor   
	clr   a                     
	clr   b
	move  #$ffff,m0
	move  m0,m1
	move  m0,m2
	move  #F__begin_init_table,r0
	move  #(-1),x1
 
_loop
 
; p:(r0)      destination start          
; p:(r0+1)    source start               
; p:(r0+2)    size of source block size  
; p:(r0+3)    size zero initialized block
 
	movem p:(r0)+,b         ; destination start
	move  b1,r1
	cmp   x1,b
	jeq _exit
 
	movem p:(r0)+,r2        ; source start           
	movem p:(r0)+,y0        ; size of expicitly initialized block
	movem p:(r0)+,y1        ; size of zero initialized block
 
	cmp   y0,a              ; (y0 == 0) ?
	jeq   _lab2
 
	do    y0,_lab2
	movem p:(r2)+,x0
	move  x0,y:(r1)+
	nop
_lab2           
	cmp   y1,a              ; (y1 == 0) ?
	jeq   _lab3
 
	do    y1,_lab3
	move  a1,y:(r1)+        ; zeros the uninitialized data memory
_lab3   
	jmp _loop
_exit
	rts                     
	endsec


;----------------------------------------------------------------------------
; LEONID working routines
;----------------------------------------------------------------------------
	section leonidwork

;----------------------------------------------------------------------------
;    SCI timer interrupt
;----------------------------------------------------------------------------
	global  sci_tim
sci_tim
	move	a0,y:<save_a0
	move	a1,y:<save_a1
	move	a2,y:<save_a2
	move	x0,y:<save_x0
	move	x1,y:<save_x1
	move	r3,y:<save_r3
	move	n3,y:<save_n3
	move	m3,y:<save_m3

	move	y:<m3_sci,m3
	move	y:<n3_sci,n3
; increment timer
	move	#-1,m3		    	; yes, set next time
	move	y:<timcnt,r3

; check if destination time reached
	move	y:<timval,a1
	move	r3,x0
	eor	x0,a	    (r3)+
	jne	_scit1
	bclr	#timer,y:<flags		; yes, clear flag bit

; check if watch-dog interval reached
_scit1	move	y:<timchg,a1
	eor	x0,a	    r3,y:<timcnt
	jne	scite
	copled	chg
	move	(r3)+n3
	nop
	move	r3,y:<timchg

scite	move	#buflen-1,m3
	move	m3,y:<m3_sci

	move	y:<save_a0,a0
	move	y:<save_a1,a1
	move	y:<save_a2,a2
	move	y:<save_x0,x0
	move	y:<save_x1,x1
	move	y:<save_r3,r3
	move	y:<save_n3,n3
	move	y:<save_m3,m3

	rti

	endsec

;----------------------------------------------------------------------------
;	 Request timer
; delay in x0 (in 1/baud s)
;----------------------------------------------------------------------------
	section req_timer
	global  stimer
stimer	move	y:<timcnt,a
	add	x0,a	    #>$00ffff,x0
	and	x0,a
	move	a1,y:<timval
	bset	#timer,y:<flags
	rts

	endsec

;****************************************************************************
;	Initialization program for EVM56002 to communicate with CS4215
;  	input connected to Mic inputs
;
;	Copywrite (c) MOTOROLA 1994
;		      Semiconductor Products Sector 
;		      Digital Signal Processing Division
;
;****************************************************************************
;
;      portc usage:
;     	bit8: SSI TX (from DSP to Codec)
;	bit7:
;	bit6:
;	bit5:
;	bit4: codec reset (from DSP to Codec)
;	bit3:
;     	bit2: data/control bar
;             0=control
;             1=data
;
;
;  PROGRAM OUTLINE:
;
;1 program fsync and sclk == output
;2 write pc0 = 0 (control mode)
;3 send 64 bit frame x times, with dcb bit = 0, keep doing until read back as 0
;4 send 64 bit frame x times, with dcb bit = 1, keep doing until read back as 1
;5 re-program fsync and sclk == input
;6 write pc0 = 1 (data mode)
;7 receive/send data (echo slots 1,2,3,4; slots 5,6,7,8 == DIP switched)
; 
;****************************************************************************

NO_PREAMP       equ     $100000 
LO_OUT_DRV      equ     $080000
HI_PASS_FILT    equ     $008000
SAMP_RATE_9     equ     $003800
SAMP_RATE_48    equ     $003000
SAMP_RATE_32    equ     $001800
SAMP_RATE_27    equ     $001000
SAMP_RATE_16    equ     $000800
SAMP_RATE_8     equ     $000000
STEREO          equ     $000400
DATA_8LIN       equ     $200300
DATA_8A         equ     $200200
DATA_8U         equ     $200100
DATA_16         equ     $200000
IMMED_3STATE    equ     $800000
XTAL2_SELECT    equ     $200000
BITS_64         equ     $000000
BITS_128        equ     $040000
BITS_256        equ     $080000
CODEC_MASTER    equ     $020000
CODEC_TX_OFF    equ     $010000

CTRL_WD_12      equ     NO_PREAMP+HI_PASS_FILT+SAMP_RATE_48+STEREO+DATA_16   ;CLB=0
CTRL_WD_34      equ     IMMED_3STATE+XTAL2_SELECT+BITS_64+CODEC_MASTER
CTRL_WD_56      equ     $000000
CTRL_WD_78      equ     $000000

HEADPHONE_EN    equ     $800000
LINEOUT_EN      equ     $400000
LEFT_ATTN       equ     $010000 ;63*LEFT_ATTN   = -94.5 dB, 1.5 dB steps
SPEAKER_EN      equ     $004000
RIGHT_ATTN      equ     $000100 ;63*RIGHT_ATTN  = -94.5 dB, 1.5 dB steps
MIC_IN_SELECT   equ     $100000
LEFT_GAIN       equ     $010000 ;15*LEFT_GAIN    = 22.5 dB, 1.5 dB steps
MONITOR_ATTN    equ     $001000 ;15*MONITOR_ATTN = mute,    6   dB steps
RIGHT_GAIN      equ     $000100 ;15*RIGHT_GAIN   = 22.5 dB, 1.5 dB steps
OUTPUT_SET      equ     HEADPHONE_EN+LINEOUT_EN+(LEFT_ATTN*4)
INPUT_SET       equ     MIC_IN_SELECT+(15*MONITOR_ATTN)+(RIGHT_ATTN*4)

TONE_OUTPUT     EQU     HEADPHONE_EN+LINEOUT_EN+(4*LEFT_ATTN)+(4*RIGHT_ATTN)   
TONE_INPUT      EQU     MIC_IN_SELECT+(15*MONITOR_ATTN)

;---DSP56002 on-chip peripheral addresses
PCD             equ     $FFE5
PCDDR           equ     $FFE3
PCC             equ     $FFE1
PBC             equ     $FFE0
CRA             equ     $FFEC
CRB             equ     $FFED
SSIDR           equ     $FFEF
IPR             equ     $FFFF
BCR             equ     $FFFE
SSISR           equ     $FFEE

	section CODEC_code
	org     p:
	global  ssi_rx_isr
	global  ssi_tx_isr
	global  codec_init

;***************************************************************************
;***** 			initialize the CS4215 codec                    *****
;***************************************************************************
; headphones and line out, and set up for no gain or attenuation, and no 
; monitor feedback.
;***************************************************************************
;***************************************************************************
;
;      initialize ssi -- fsync and sclk ==> outputs
;
codec_init
	movep   #$0000,x:PCC    ;  turn off ssi port 
	movep   #$4303,x:CRA    ;  40MHz/16 = 2.5MHz SCLK, WL=16 bits, 4W/F
	movep   #$FB30,x:CRB    ; RIE,TIE,RE,TE, NTWK, SYN, FSR/RSR->bit
	movep   #$14,x:PCDDR    ; setup pc2 and pc4 as outputs
	movep   #$0,x:PCD       ; D/C~ and RESET~ = 0 ==> control mode
				;----reset delay for codec ----
	do      #500,_delay_loop
	rep     #2000           ; 100 us delay
	nop
_delay_loop
	bset    #4,x:PCD        ; RESET~ = 1
	movep   #$3000,x:IPR    ; set interrupt priority level
	andi    #$FC,mr         ; enable interrupts

;*****************************************************************************
; The following data sets up the CS4215 control mode data:
;    (CTS = Control Time Slot, U/LN = upper/lower Nibble)
;
;       +------ CTS1-UN:   0      0      1     MLB      0 0 0 0 
;       |+----- CTS1-LN:  OLB    CLB     X      X       0 0 0 0 
;       ||+---- CTS2-UN:  HPF     X     DFR2   DFR1     0 0 1 0
;       |||+--- CTS2-LN:  DFR0   ST     DF1    DF0      1 1 0 0
; x0 = $002Cxx
;
;       +------ CTS3-UN:  ITS    MCK2   MCK1   MCK0     1 0 0 0
;       |+----- CTS3-LN:  BSEL1  BSEL0  XCLK   XEN      1 0 0 0 
;       ||+---- CTS4-UN:  TEST   TEST   TEST   TEST     (TEST MUST BE 0)
;       |||+--- CTS4-LN:  TEST   TEST   ENL    DAD      0 0 0 0 
; x0 = $8800xx
;*****************************************************************************

;--- set up buffer with control mode data
	move    #CTRL_WD_12,x0
	move    x0,y:<TX_BUFF_BASE
	move    #CTRL_WD_34,x0
	move    x0,y:<TX_BUFF_BASE+1
	move    #CTRL_WD_56,x0
	move    x0,y:<TX_BUFF_BASE+2
	move    #CTRL_WD_78,x0
	move    x0,y:<TX_BUFF_BASE+3

	movep   #$01E8,x:PCC    ; Turn on ssi port

;
; CLB == 0
;
	jclr    #3,x:SSISR,*            ; wait until rx frame bit==1
	jset    #3,x:SSISR,*            ; wait until rx frame bit==0
	jclr    #3,x:SSISR,*            ; wait until rx frame bit==1
;	jset    #18,y:<RX_BUFF_BASE,*   ; loop until CLB set
_wt1	move	y:<RX_BUFF_BASE,a0
	jset    #18,a,_wt1

;
; CLB == 1
;
	bset    #18,y:<TX_BUFF_BASE	; set CLB
	do      #4,_init_loopB
	jclr    #2,x:SSISR,*            ; wait until tx frame bit==1
	jset    #2,x:SSISR,*            ; wait until tx frame bit==0
_init_loopB
	movep   #0,x:PCC                ; disable, reset SSI

;*****************************************************************************
;  Now CLB should be 1 -- re-program fsync and sclk direction (i/p) -- also,
;  circular buffer pointers for echoing data r0=current, r1=old data to send
;  1 frame later
;
	movep   #$4303,x:CRA    ; 16bits,4 word/frame, /2/4/2=2.5 MHz
	movep   #$FB00,x:CRB    ; rcv,xmt & int ena,netwk,syn,sclk==i/p,msb 1st
	movep   #$14,x:PCD      ; D/C~ pin = 1  ==> data mode
	movep   #$01E8,x:PCC    ;  turn on ssi port
	rts

;************************** SSI TRANSMIT ISR **************************
ssi_tx_isr
	move    r0,y:<r0_save 		; Save r0 to the stack.
	move    m0,y:<m0_save           ; Save m0 to the stack.
	move    #3,m0                   ; Modulus 4 buffer.
	move    y:TX_PTR,r0             ; Load the pointer to the tx buffer.
	jclr    #2,x:SSISR,_next_tx      ; If not frame sync, jump to transmit data.
	move    #TX_BUFF_BASE+1,r0      ; If frame sync, reset pointer.
	nop
_next_tx
	movep   y:(r0)+,x:SSIDR         ; SSI transfer data register.
	move    r0,y:<TX_PTR            ; Update tx buffer pointer.
	move    y:<m0_save,m0           ; Restore m0.
	move    y:<r0_save,r0           ; Restore r0.
	rti

;************************** SSI receive ISR**************************
ssi_rx_isr
	move    r0,y:<r0_save    	; Save r0 to the stack.
	move    m0,y:<m0_save           ; Save m0 to the stack.
	move    #3,m0                   ; Modulo 4 buffer.
	move    y:<RX_PTR,r0            ; Load the pointer to the rx buffer.
	jclr    #3,x:SSISR,_next_rx      ; If not fr. syc, jump to receive data.
	move    #RX_BUFF_BASE,r0        ; If frame sync, reset base pointer.
	nop
_next_rx
	movep   x:SSIDR,y:(r0)+         ; Read out received data to buffer.
	move    r0,y:<RX_PTR            ; Update rx buffer pointer.
	move    y:<m0_save,m0           ; Restore m0.
	move    y:<r0_save,r0           ; Restore r0.
	rti

	endsec

