/*--------------------------------------------------------------------------- 
 * Programming the EVM56002DSP in C
 *	Program Example TEST3.C
 * This example reads toggle switches (PB8-PB11) and displays that on 
 * LED's (PB0-PB3). The CODEC runs the "talk thru" task in real time.
 *
 * Author: Johan Forrer, KC7WW
 * Date: January 1997.
 *--------------------------------------------------------------------------*/

#include <stdio.h>

/*--------------------------------------------------------------------------- 
 * Global Definitions to Real-Time Kernel variables  
 * Note this format for Externals is very IMPORTANT!
 *--------------------------------------------------------------------------*/
extern unsigned int Mr __asm("flags_mirror");

/*--------------------------------------------------------------------------- 
 * Globals for this program 
 * PB reflects the status of a Read of the 56002's PB port
 *--------------------------------------------------------------------------*/
unsigned int PB;

/*--------------------------------------------------------------------------- 
 * Function Prototypes 
 *--------------------------------------------------------------------------*/
int Get_PB(void);
int Set_PB(unsigned int);
void WaitForCodec(void);

void main()
{

	for(;;) {
		WaitForCodec();				 /* Wait for an audio sample		*/
		Get_PB();						 /* Update PB 							*/
		Set_PB(((PB>>8)&0xF)|Mr);	 /* Toggle switches + bits 13/14	*/
		}
}





/*========================HELPER ROUTINES===================================*/
/*---------------------------------------------------------------------------
 * Routine writes to LED's B0 - B3
 * Note how the argument is taken off the stack,
 * also that all registers are saved. 
 * bits 12/13 is left intact.
 *--------------------------------------------------------------------------*/
Set_PB(unsigned int val)
{
	__asm volatile("	move	n0,Y:(r6)+
							move	#-3,n0\n\
							move 	Y:(r0+n0),a\n\
							move	a1,X:$FFE4\n\
							move	Y:-(r6),n0");
}

/*---------------------------------------------------------------------------
 * Routine reads PB and returns value in global variable PB
 *--------------------------------------------------------------------------*/
Get_PB()
{
	__asm volatile(" 	move	X:$FFE4,a\n\
							move	a1,Y:FPB");

}

/*---------------------------------------------------------------------------
 * We wait here for a CODEC sample to appear.  Processing in the main loop 
 * is at the CODEC sample rate, i.e., 48kSPS - a lot a wasted time ! 
 * It would be better to buffer data and use an interrupt-based  
 * mechanism to interrupt the main loop when	a sample/buffer is ready.
 *--------------------------------------------------------------------------*/
void WaitForCodec()
{
		__asm volatile("	jset    #2,x:$FFEE,*\n\
								jclr    #2,x:$FFEE,*\n\
								move    y:<RX_BUFF_BASE,a\n\
								move    a,y:<TX_BUFF_BASE\n\
								move    y:<RX_BUFF_BASE+1,a\n\
								move    b,y:<TX_BUFF_BASE+1");
}
